import numpy as np
from .source import Source


class Spectrum:

    def __init__(self, fps, decay, channel_count=2, sample_rate=44100,):
        self.sample = Source(channel_count, sample_rate)
        self.decay = decay

        buffer_size = sample_rate // fps
        self.buffer_size = buffer_size

        self.history = np.zeros(
            (channel_count, buffer_size * 8), dtype='int16')
        self.history_index = 0

        self.min_sample = 10
        self.max_sample = self.min_sample

    def stop(self):
        self.sample.stop()

    def start(self):
        self.sample.start()

    def updateHistory(self):

        data = self.sample.read()
        data = np.fromstring(data, 'int16')

        len_data = len(data) // self.history.shape[0]

        len_history = self.history.shape[1]
        index = self.history_index
        assert len_data < len_history

        data = data.reshape((len_data, self.history.shape[0]))
        data = np.rollaxis(data, 1)

        if index + len_data > len_history:
            self.history[:, index:] = data[:, :len_history - index]
            self.history[:, :index + len_data -
                         len_history] = data[:, len_history - index:]
            self.history_index -= len_history
        else:
            self.history[:, index:index + len_data] = data
        self.history_index += len_data

        data_history = np.concatenate([
            self.history[:, self.history_index:],
            self.history[:, :self.history_index],
        ], axis=1)

        return data_history

    def getData(self):
        data_history = self.updateHistory()

        fft_freq = []

        def fun(start, end,  rel):
            size = self.buffer_size
            start = int(start * rel)
            end = int(end * rel)
            size = int(size * rel)
            d = data_history[:, -size:]

            fft = np.absolute(np.fft.rfft(d, n=size))
            fft_freq.insert(0, fft[:, start:end])

        # higher resolution and latency for lower frequency
        fun(110, 150, 2)
        fun(80, 110, 3)
        fun(50, 80, 4)
        fun(30, 50, 5)
        fun(10, 30, 6)
        fun(0, 10, 8)

        fft = np.concatenate(fft_freq, axis=1)

        exp = 2
        retain = (1 - self.decay)**exp
        decay = 1 - retain

        vol = self.min_sample + np.mean(fft ** exp)
        self.max_sample = self.max_sample * retain + vol * decay
        return fft / self.max_sample ** (1 / exp)



import gi
gi.require_version('Gtk', '3.0')
from gi.repository import Gtk, Gdk
import sys
from .visualizer import Visualizer
from .shortcut import Shortcut
from .taskbar import Taskbar
import logging
from .multiload import Multiload
from . import config
from . import helper

if config.log == 'debug':
    logging.basicConfig(level=logging.DEBUG)

style_provider = Gtk.CssProvider()
style_provider.load_from_data(config.style_sheet.encode())

Gtk.StyleContext.add_provider_for_screen(
    Gdk.Screen.get_default(),
    style_provider,
    Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION)


class Panel:

    def destroy(self, window):
        Gtk.main_quit()

    def __init__(self, display):
        self.background_color = helper.color(config.background_color)

        self.display = display                   # Display obj
        self.screen = display.screen()          # Screen obj

        self.window_gtk = Gtk.Window()
        self.setGtkProps(self.window_gtk)
        self.window_gtk.show()

        self.winid = self.window_gtk.get_window().get_xid()
        self.window_xlib = display.create_resource_object('window', self.winid) 
        self.setProps(self.display, self.window_xlib)
        self.setStruts(self.window_xlib)
        self.display.flush()

        self.box = Gtk.Box(Gtk.Orientation.HORIZONTAL, 0)
        #self.box.override_background_color(Gtk.StateType.NORMAL, self.background_color)
        if config.fake_shadow:
            ctx = self.box.get_style_context()
            if config.position == 'bottom':
                ctx.add_class('fake-shadow-top')
            elif config.position == 'top':
                ctx.add_class('fake-shadow-bottom')

        self.window_gtk.add(self.box)
        for section in config.sections:
            if section == 'visualizer':
                self.visualizer = Visualizer(
                    background_color=config.visualizer_background, 
                    padding=config.visualizer_padding)
                self.box.pack_start(self.visualizer, True, True, 0)
            elif section == 'taskbar':
                self.taskbar = Taskbar(self.display,
                                       background_color=self.background_color,
                                       size=config.height,)
                self.box.pack_start(self.taskbar, False, False, 0)
            elif type(section) is not str:
                self.box.pack_start(Shortcut(
                    background_color=self.background_color,
                    size=config.height, shortcut=section), False, False, 0)
            elif section == 'multiload':
                colors = {'cpu': {'background': config.multiload_cpu_background,
                                  'foreground': config.multiload_cpu_foreground, },
                          'mem': {'background': config.multiload_mem_background,
                                  'foreground': config.multiload_mem_foreground, },
                          'net': {'background': config.multiload_net_background,
                                  'foreground': config.multiload_net_foreground, },
                          'disk': {'background': config.multiload_disk_background,
                                   'foreground': config.multiload_disk_foreground, }, }
                w = config.height * 2 * \
                    config.multiload_layout[0] // config.multiload_layout[1]

                self.multiload_cpu = Multiload(
                    self.background_color,
                    colors, w, config.height,
                    fake_shadow=config.multiload_fake_shadow,
                    interval=config.multiload_interval,
                    layout=config.multiload_layout,
                    inner_gap=config.multiload_inner_gap,
                    outer_gap=config.multiload_outer_gap,
                )
                self.box.pack_start(self.multiload_cpu, False, False, 0)
        # menu
        self.create_menu()
        # show all
        self.window_gtk.connect_after('destroy', self.destroy)
        self.window_gtk.show_all()

    def addTime(self, expand, fill, padding):
        label = Gtk.Label()
        label.set_text("time")
        self.box.pack_start(label, expand, fill, padding)

    def quit(self, e):
        self.multiload_cpu.destory()
        self.visualizer.destory()
        Gtk.main_quit()

    def create_menu(self):
        self.menu = Gtk.Menu()
        i1 = Gtk.MenuItem("Quit")
        i1.connect("activate", self.quit)
        self.menu.append(i1)
        self.window_gtk.connect('button-release-event',
                                lambda b, e: e.button == 3
                                and self.menu.popup(
                                    None, None, None, None, 0,
                                    Gtk.get_current_event_time()))
        self.menu.show_all()

    def setGtkProps(self, win):
        win.set_app_paintable(True)
        screen = win.get_screen()
        rgba = screen.get_rgba_visual()
        win.set_visual(rgba)
        win.set_wmclass("panon","Panon" )
        win.set_name("panon")
        win.set_type_hint(Gdk.WindowTypeHint.DOCK)
        win.set_decorated(False)
        win.stick()
        win.set_keep_above(True)
        win.fullscreen()

    def setProps(self, dsp, win):
        #----------------------------
        """ Set necessary X atoms and panel window properties """
        self._ABOVE = dsp.intern_atom("_NET_WM_STATE_ABOVE")
        self._BELOW = dsp.intern_atom("_NET_WM_STATE_BELOW")
        self._BLACKBOX = dsp.intern_atom("_BLACKBOX_ATTRIBUTES")
        self._CHANGE_STATE = dsp.intern_atom("WM_CHANGE_STATE")
        self._CLIENT_LIST = dsp.intern_atom("_NET_CLIENT_LIST")
        self._CURRENT_DESKTOP = dsp.intern_atom("_NET_CURRENT_DESKTOP")
        self._DESKTOP = dsp.intern_atom("_NET_WM_DESKTOP")
        self._DESKTOP_COUNT = dsp.intern_atom("_NET_NUMBER_OF_DESKTOPS")
        self._DESKTOP_NAMES = dsp.intern_atom("_NET_DESKTOP_NAMES")
        self._HIDDEN = dsp.intern_atom("_NET_WM_STATE_HIDDEN")
        self._ICON = dsp.intern_atom("_NET_WM_ICON")
        self._NAME = dsp.intern_atom("_NET_WM_NAME")
        self._RPM = dsp.intern_atom("_XROOTPMAP_ID")
        self._SHADED = dsp.intern_atom("_NET_WM_STATE_SHADED")
        self._SHOWING_DESKTOP = dsp.intern_atom("_NET_SHOWING_DESKTOP")
        self._SKIP_PAGER = dsp.intern_atom("_NET_WM_STATE_SKIP_PAGER")
        self._SKIP_TASKBAR = dsp.intern_atom("_NET_WM_STATE_SKIP_TASKBAR")
        self._STATE = dsp.intern_atom("_NET_WM_STATE")
        self._STICKY = dsp.intern_atom("_NET_WM_STATE_STICKY")
        self._STRUT = dsp.intern_atom("_NET_WM_STRUT")
        self._STRUTP = dsp.intern_atom("_NET_WM_STRUT_PARTIAL")
        self._WMSTATE = dsp.intern_atom("WM_STATE")

        P_WIDTH = self.screen.width_in_pixels

        win.set_wm_name("panon")
        win.set_wm_class("panon", "Panon")
        # win.set_wm_hints(flags=(Xutil.InputHint | Xutil.StateHint),
        #                 input=0, initial_state=1)
        win.set_wm_normal_hints(flags=(
            Xutil.PPosition | Xutil.PMaxSize | Xutil.PMinSize),
            min_width=P_WIDTH, min_height=config.height,
            max_width=P_WIDTH, max_height=config.height)
        # win.change_property(dsp.intern_atom("_WIN_STATE"),
        #                    Xatom.CARDINAL, 32, [1])
        # win.change_property(dsp.intern_atom("_MOTIF_WM_HINTS"),
        #                    dsp.intern_atom("_MOTIF_WM_HINTS"), 32, [0x2, 0x0, 0x0, 0x0, 0x0])
        #win.change_property(self._DESKTOP, Xatom.CARDINAL, 32, [0xffffffff])
        # win.change_property(dsp.intern_atom("_NET_WM_WINDOW_TYPE"),
        #                    Xatom.ATOM, 32, [dsp.intern_atom("_NET_WM_WINDOW_TYPE_DOCK")])

    #----------------------------------
    def setStruts(self, win, hidden=0):
        #----------------------------------
        """ Set the panel struts according to the state (hidden/visible) """

        P_WIDTH = self.screen.width_in_pixels
        if config.position == 'bottom':
            P_LOCATION = self.screen.height_in_pixels - config.height
        elif config.position == 'top':
            P_LOCATION = 0
        P_START = 0

        win.configure(y=P_LOCATION, height=config.height)

        if P_LOCATION == 0:
            # top
            if not hidden:
                top = config.height
            else:
                top = HIDDEN_SIZE

            top_start = P_START
            top_end = P_START + P_WIDTH
            bottom = bottom_start = bottom_end = 0
        else:
            # bottom
            top = top_start = top_end = 0
            if not hidden:
                bottom = config.height
            else:
                bottom = HIDDEN_SIZE

            bottom_start = P_START
            bottom_end = P_START + P_WIDTH

        win.change_property(self._STRUT, Xatom.CARDINAL,
                            32, [0, 0, top, bottom])
        win.change_property(self._STRUTP, Xatom.CARDINAL, 32, [0, 0, top, bottom,
                                                               0, 0, 0, 0, top_start, top_end, bottom_start, bottom_end])


from .singleton import Singleton
from threading import Thread
import os
from Xlib import X, display, error, Xatom, Xutil
import Xlib.protocol.event
def main():
    SOCKET_FILE = "/run/user/%s/gtk_visualizer.socket" % os.getuid()
    sin = Singleton(SOCKET_FILE)
    if sin.start():
        # Thread(target=sin.loop).start()
        p = Panel(display.Display())
        import signal
        signal.signal(signal.SIGINT, signal.SIG_DFL)
        Gtk.main()

if __name__=='__main__':
    main()



import gi
import math
gi.require_version('Gtk', '3.0')
from gi.repository import Gtk, Gdk


def color(_str):
    f = _str[1:]
    f = int(f[:2], 16), int(f[2:4], 16), int(f[4:6], 16), int(f[6:8], 16)
    f = f[0] / 256, f[1] / 256, f[2] / 256, f[3] / 256

    rgba = Gdk.RGBA(
        red=f[0],
        green=f[1],
        blue=f[2],
        alpha=f[3],)
    return rgba
    return f


def hsv2rgb(h, s, v):
    h = float(h)
    s = float(s)
    v = float(v)
    h60 = h / 60.0
    h60f = math.floor(h60)
    hi = int(h60f) % 6
    f = h60 - h60f
    p = v * (1 - s)
    q = v * (1 - f * s)
    t = v * (1 - (1 - f) * s)
    r, g, b = 0, 0, 0
    if hi == 0:
        r, g, b = v, t, p
    elif hi == 1:
        r, g, b = q, v, p
    elif hi == 2:
        r, g, b = p, v, t
    elif hi == 3:
        r, g, b = p, q, v
    elif hi == 4:
        r, g, b = t, p, v
    elif hi == 5:
        r, g, b = v, p, q
    r, g, b = int(r * 255), int(g * 255), int(b * 255)
    return r, g, b


def rgb2hsv(r, g, b):
    r, g, b = r / 255.0, g / 255.0, b / 255.0
    mx = max(r, g, b)
    mn = min(r, g, b)
    df = mx - mn
    if mx == mn:
        h = 0
    elif mx == r:
        h = (60 * ((g - b) / df) + 360) % 360
    elif mx == g:
        h = (60 * ((b - r) / df) + 120) % 360
    elif mx == b:
        h = (60 * ((r - g) / df) + 240) % 360
    if mx == 0:
        s = 0
    else:
        s = df / mx
    v = mx
    return h, s, v



import gi
gi.require_version('Gtk', '3.0')
from gi.repository import Gtk, Gdk, GObject, Pango
import os
import subprocess


class Shortcut(Gtk.EventBox):
    def __init__(self, background_color, size, shortcut):
        super(Shortcut, self).__init__()

        self.override_background_color(Gtk.StateType.NORMAL, background_color)
        self.shortcut = shortcut

        if 'icon-name' in shortcut:
            icon = Gtk.Image.new_from_icon_name(shortcut['icon-name'], size)
            self.add(icon)
        if 'auto-command' in shortcut:
            self.label = Gtk.Label()
            self.label.set_single_line_mode(True)
            self.label.set_ellipsize(Pango.EllipsizeMode.END)
            if 'max-width' in shortcut:
                w = shortcut['max-width']
                self.label.set_max_width_chars(w)
            ctx = self.label.get_style_context()
            ctx.add_class('label')
            self.add(self.label)
            self.tick()
            GObject.timeout_add(1000 * shortcut['interval'], self.tick)

        self.add_events(Gdk.EventMask.SCROLL_MASK)
        self.connect('scroll-event', self.do_scroll_event)
        self.connect('button-release-event', self.do_button_release_event)

    def tick(self):
        text = subprocess.check_output(
            self.shortcut['auto-command'],
            shell=True)
        text = text.decode()
        text = text.strip()
        self.label.set_text(text)
        return True

    def do_scroll_event(self, widget, e):
        if e.direction == Gdk.ScrollDirection.UP:
            if 'scroll-up' in self.shortcut:
                os.system(self.shortcut['scroll-up'])
            return True
        elif e.direction == Gdk.ScrollDirection.DOWN:
            if 'scroll-down' in self.shortcut:
                os.system(self.shortcut['scroll-down'])
            return True

    def do_button_release_event(self, widget, event):
        if event.button == 1:
            if 'click' in self.shortcut:
                os.system(self.shortcut['click'])
            return True
        if event.button == 3:
            if 'rightclick' in self.shortcut:
                os.system(self.shortcut['rightclick'])
            return True



import gi
gi.require_version('Gtk', '3.0')
from gi.repository import Gtk, GObject
import cairo
from .. import helper
from .. import config
from .fallback import VisualizerCairo
from .opengl import VisualizerGL
from .spectrum import Spectrum
from queue import Queue
from threading import Thread


class Visualizer(Gtk.EventBox):
    stop = False
    stop_gen_data = False

    def tick(self):
        self.queue_draw()
        return True  # Causes timeout to tick again.

    def destory(self):
        self.stop_gen_data = True

    def __init__(self, background_color, fps=60, padding=4, use_opengl=False):
        super(Visualizer, self).__init__()
        self.background_color = helper.color(background_color)
        self.data_queue = Queue(3)

        self.padding = padding
        self.spectrum = Spectrum(fps,  config.visualizer_decay)
        GObject.timeout_add(1000 // fps, self.tick)

        self.use_opengl = use_opengl

        if use_opengl:
            Thread(target=self.run).start()
            self.da = VisualizerGL(self.getData)
            ol = Gtk.Overlay()
            ol.add(self.da)
            label = Gtk.Label()
            label.set_text("111111")
            label.show()
            # ol.add(label)
            label = Gtk.Label()
            label.set_text("bbbbbbbbbHHHHHH")
            label.show()
            ol.add(label)
            self.override_background_color(
                Gtk.StateType.NORMAL, self.background_color)
            self.da = ol
        else:
            self.da = VisualizerCairo(
                self.background_color, self.getData, self.padding)
        self.add(self.da)
        self.connect('button-release-event', self.do_button_release_event)

    def do_button_release_event(self, widget, event=None):
        if event and event.button == 1:
            if not self.stop:
                self.da.stop()
                self.spectrum.stop()
                self.stop = True
            else:
                self.stop = False
                self.spectrum.start()
                self.da.start()
            return True
        else:
            return False

    def run(self):
        while not self.stop_gen_data:
            self.data_queue.put(self.__getData())

    def getData(self):
        if self.use_opengl:
            return self.data_queue.get()
        else:
            return self.spectrum.getData()



from OpenGL.GLU import *
from OpenGL.GL import *
from OpenGL import GL
import gi
gi.require_version('Gtk', '3.0')
from gi.repository import Gtk, Gdk, GObject
from OpenGL.GL import shaders
from OpenGL.raw.GL.ARB.vertex_array_object import glGenVertexArrays, \
    glBindVertexArray
import numpy as np
from PIL import Image
from .. import glsl


class VisualizerGL(Gtk.GLArea):
    empty = False

    def __init__(self, getData):
        super(VisualizerGL, self).__init__()
        self.getData = getData
        self.set_required_version(3, 3)
        # TODO This line brings terrible performance.
        self.set_has_alpha(True)
        self.connect('realize', self.on_configure_event)
        self.connect('render', self.on_draw)
        self.set_double_buffered(False)

    def stop(self):
        self.empty = True

    def start(self):
        self.empty = False

    def loadTex(self, data):
        img_data = np.zeros((3, data.shape[0], 4), dtype='uint8')
        img_data[:, :, 0] = data * 256
        image =  Image.fromarray(img_data)
        width = image.size[0]
        height = image.size[1]
        image_bytes = image.convert("RGBA").tobytes("raw", "RGBA", 0, -1)
        gluBuild2DMipmaps(GL_TEXTURE_2D, GL_RGBA, width, height,
                          GL_RGBA, GL_UNSIGNED_BYTE, image_bytes)

    def on_configure_event(self, widget):
        widget.make_current()
        context = widget.get_context()
        vs = shaders.compileShader(
            glsl.load('./vert.glsl'), GL.GL_VERTEX_SHADER)
        fs = shaders.compileShader(
            glsl.load('./frag.glsl'), GL.GL_FRAGMENT_SHADER)
        self.program = shaders.compileProgram(vs, fs)

        self.vertex_array_object = GL.glGenVertexArrays(1)
        GL.glBindVertexArray(self.vertex_array_object)
        vertex_buffer = GL.glGenBuffers(1)
        GL.glBindBuffer(GL.GL_ARRAY_BUFFER, vertex_buffer)

        position = GL.glGetAttribLocation(self.program, 'a_position')

        GL.glEnableVertexAttribArray(position)
        GL.glVertexAttribPointer(
            position, 4, GL.GL_FLOAT, False, 0, ctypes.c_void_p(0))

        vertices = [
            1,  1, 0, 1,
            -1,  1, 0, 1,
            -1, -1, 0, 1,
            1,  1, 0, 1,
            1,  -1, 0, 1,
            -1, -1, 0, 1,
        ]

        vertices = np.array(vertices, dtype=np.float32)

        GL.glBufferData(GL.GL_ARRAY_BUFFER, 96,
                        vertices, GL.GL_STATIC_DRAW)
        GL.glBindVertexArray(0)
        GL.glDisableVertexAttribArray(position)
        GL.glBindBuffer(GL.GL_ARRAY_BUFFER, 0)
        self.tex1 = glGenTextures(1)
        return True

    def on_draw(self, widget, *args):
        if self.empty:
            return
        data = self.getData()
        GL.glBindTexture(GL_TEXTURE_2D, self.tex1)
        self.loadTex(data)
        widget.attach_buffers()
        GL.glClear(GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT)
        GL.glUseProgram(self.program)
        GL.glBindVertexArray(self.vertex_array_object)
        GL.glBindTexture(GL_TEXTURE_2D, self.tex1)
        GL.glDrawArrays(GL.GL_TRIANGLES, 0, 6)
        GL.glBindVertexArray(0)
        GL.glUseProgram(0)
        GL.glFlush()
        return True



import cairo
import gi
gi.require_version('Gtk', '3.0')
from gi.repository import Gtk, Gdk
from .. import helper


class VisualizerCairo(Gtk.DrawingArea):
    empty = False

    def __init__(self, background_color, getData, padding):
        super(VisualizerCairo, self).__init__()
        self.hue_gradient_position = 0
        self.padding = padding
        self.background_color = background_color
        self.getData = getData
        self.update_hue_gradient()
        self.add_events(Gdk.EventMask.SCROLL_MASK)
        self.connect('scroll-event', self.do_scroll_event)
        self.connect('draw', self.do_draw_cb)

    def stop(self):
        self.empty = True

    def start(self):
        self.empty = False

    def do_scroll_event(self, widget, e):
        if type(self.sources[0][1]) is Gdk.RGBA:
            pass
        else:
            if e.direction == Gdk.ScrollDirection.UP:
                self.hue_gradient_position += 0.02
                self.update_hue_gradient()
                return True
            elif e.direction == Gdk.ScrollDirection.DOWN:
                self.hue_gradient_position -= 0.02
                self.update_hue_gradient()
                return True

    def update_hue_gradient(self):
        self.sources = [
            (4, self.create_gradient(alpha=0.1, position=self.hue_gradient_position)),
            (3, self.create_gradient(alpha=0.2, position=self.hue_gradient_position)),
            (2, self.create_gradient(alpha=0.3, position=self.hue_gradient_position)),
            (1, self.create_gradient(alpha=0.5, position=self.hue_gradient_position)),
            (0.5, self.create_gradient(alpha=1, position=self.hue_gradient_position)),
        ]

    def create_gradient(self, alpha=0.8, position=0, width=800, hue_step=60, hue_start=180):
        hue_gradient = cairo.LinearGradient(0.0, 0.0, width, 0)
        for hue in range(0, 360, hue_step):
            rgb = helper.hsv2rgb((hue + hue_start) % 360, 1, alpha)
            hue_gradient.add_color_stop_rgba(
                (hue / 360 + position) % 1, *rgb, alpha)
        hue_gradient.set_extend(cairo.EXTEND_REPEAT)
        return hue_gradient

    def do_draw_cb(self, widget, cr):
        alloc = self.get_allocation()
        w, h = alloc.width, alloc.height

        cr.set_source_rgba(*self.background_color)
        cr.rectangle(0, 0, w, h)
        cr.fill()

        if self.empty:
            return
        bins = self.getData()
        assert bins.shape[0]==2
        x, y = self.padding, self.padding
        w, h = w - 2 * self.padding, h - 2 * self.padding

        for rel, source in self.sources:
            if type(source) is Gdk.RGBA:
                cr.set_source_rgba(*source)
            else:
                cr.set_source(source)
            cr.set_operator(cairo.OPERATOR_SOURCE)
            cr.move_to(x + 0, y + h / 2)  # middle left
            width =  w / bins.shape[1]
            for i in range(bins.shape[1]):
                height = rel * h * bins[0,i]
                cr.line_to(x + i * width, y + h / 2 - height / 2)
            cr.line_to(x + w, y + h / 2)  # middle right
            cr.close_path()

            cr.move_to(x + 0, y + h / 2)  # middle left
            for i in range(bins.shape[1]):
                height = rel * h * bins[1,i]
                cr.line_to(x + i * width, y + h / 2 + height / 2)
            cr.line_to(x + w, y + h / 2)  # middle right
            cr.close_path()
            cr.fill()



import numpy as np


class GLSpectrum:
    def __init__(self, sample, buffer_size, decay):
        self.sample = sample
        self.decay = decay
        self.history = [[]] * 8
        self.buffer_size = buffer_size
        self.min_sample = 10
        self.max_sample = self.min_sample

        from .glfft import GLFFT
        self.glfft = GLFFT()

    def getData(self):
        data = self.sample.read()
        data = np.fromstring(data, 'int16')

        self.history.append(data)
        if sum([len(d) for d in self.history[1:]]) > self.buffer_size * 8:
            self.history.pop(0)

        data_history = np.concatenate(self.history)
        fft_freq = []

        def fun(start, end,  rel):
            size = self.buffer_size
            if rel > 20:
                start, end = int(start), int(end)
                rel = int(rel)
                d = data_history[-size * rel:].reshape((rel, size))
                d = np.mean(d, axis=0)
            else:
                start = int(start * rel)
                end = int(end * rel)
                size = int(size * rel)
                d = data_history[-size:]

            fft = self.glfft.compute(d.astype('float32').tobytes())
            fft = np.frombuffer(fft, dtype='float32')
            end = min(len(fft) // 2, end)
            fft_freq.insert(0, fft[start:end])
            fft_freq.append(fft[len(fft) - end:len(fft) - start])
        # higher resolution and latency for lower frequency

        sections = 8
        r = 0.6
        rels = 8 * r**np.arange(sections)
        start = 0
        sections = []
        for rel, freq_width in zip(rels, len(data) * 1 / rels / sum(1 / rels) // 4):
            if rel > 2:
                freq_width *= rel
                pass
            sections.append((start, start + freq_width, rel))
            start += freq_width
        sections.reverse()
        for start, end, rel in sections:
            #fun(start, end, rel)
            pass

        fun(110, 150, 2)
#        fun(0, 110, 3)
        fun(80, 110, 3)
        fun(50, 80, 4)
        fun(30, 50, 5)
        fun(10, 30, 6)
        fun(0, 10, 8)

        fft = np.concatenate(fft_freq)

        exp = 2
        retain = (1 - self.decay)**exp
        decay = 1 - retain

        vol = self.min_sample + np.mean(fft ** exp)
        self.max_sample = self.max_sample * retain + vol * decay
        bins = fft / self.max_sample ** (1 / exp)
        return bins



import gi
gi.require_version('Gtk', '3.0')
from gi.repository import Gtk, Gdk, GObject
from ewmh import EWMH
from logging import getLogger
from Xlib import X, Xutil, Xatom
import Xlib
import cairo
import os
ewmh = EWMH()

childLogger = getLogger(__name__)


class Taskbar(Gtk.EventBox):

    def __init__(self, display, size,
                 background_color,
                 fps=60, animation_duration=0.1):
        super(Taskbar, self).__init__()
        self.display = display
        self.screen = display.screen()          # Screen obj
        self.root = self.screen.root          # Display root

        self.setup_atom()
        self.size = size
        self.fps = fps
        self.background_color = background_color
        self.animation_duration = animation_duration
        self.tasks_draw = []

        self.icontheme = Gtk.IconTheme()
        self.icontheme.set_screen(self.get_screen())

        lst = self.root.get_full_property(
            self._CLIENT_LIST, Xatom.WINDOW).value
        for _id in lst:
            self.addTask(_id)

        self.da = Gtk.DrawingArea()
        self.da.connect('draw', self.do_draw_cb)
        self.add(self.da)

        self.create_menu()

        self.add_events(Gdk.EventMask.SCROLL_MASK)
        self.connect('button-release-event', self.do_button_release_event)
        self.connect('scroll-event', self.do_scroll_event)

        root = self.display.screen().root
        root.change_attributes(event_mask=X.SubstructureNotifyMask)
        GObject.timeout_add(200, self.tick)
        GObject.timeout_add(1000 // self.fps, self.update)

    def do_button_release_event(self, widget, event):
        if event.button == 1:
            self.toggleWindow(self.getTask(event))
            return True
        elif event.button == 3:
            self.target_task = self.getTask(event)
            self.menu.popup(None, None, None, None, 0,
                            Gtk.get_current_event_time())
            return True

    def do_scroll_event(self, widget, e):
        if e.direction == Gdk.ScrollDirection.UP:
            self.activateWindow(self.getTask(e), offset=1)
            return True
        elif e.direction == Gdk.ScrollDirection.DOWN:
            self.activateWindow(self.getTask(e), offset=-1)
            return True

    def close_all(self, event):
        for win in self.target_task.windows:
            os.system('wmctrl -i -c %s' % win.id)
            # win.destroy()
        return True

    def setup_atom(self):
        self._NET_WM_STATE_SKIP_TASKBAR = self.display.intern_atom(
            '_NET_WM_STATE_SKIP_TASKBAR')
        self._NET_WM_WINDOW_TYPE_DOCK = self.display.intern_atom(
            '_NET_WM_WINDOW_TYPE_DOCK')
        self._NET_WM_WINDOW_TYPE_NORMAL = self.display.intern_atom(
            '_NET_WM_WINDOW_TYPE_NORMAL')
        self._NET_WM_WINDOW_TYPE_MENU = self.display.intern_atom(
            '_NET_WM_WINDOW_TYPE_MENU')
        self._NET_WM_WINDOW_TYPE_SPLASH = self.display.intern_atom(
            '_NET_WM_WINDOW_TYPE_SPLASH')
        self._NET_WM_WINDOW_TYPE_TOOLBAR = self.display.intern_atom(
            '_NET_WM_WINDOW_TYPE_TOOLBAR')
        self._NET_WM_WINDOW_TYPE_UTILITY = self.display.intern_atom(
            '_NET_WM_WINDOW_TYPE_UTILITY')
        self._NET_WM_STATE_HIDDEN = self.display.intern_atom(
            "_NET_WM_STATE_HIDDEN")
        self.WM_CHANGE_STATE = self.display.intern_atom("WM_CHANGE_STATE")

        dsp = self.display
        self._CLIENT_LIST = dsp.intern_atom("_NET_CLIENT_LIST")

        self._ABOVE = dsp.intern_atom("_NET_WM_STATE_ABOVE")
        self._BELOW = dsp.intern_atom("_NET_WM_STATE_BELOW")
        self._BLACKBOX = dsp.intern_atom("_BLACKBOX_ATTRIBUTES")
        self._CHANGE_STATE = dsp.intern_atom("WM_CHANGE_STATE")
        self._CLIENT_LIST = dsp.intern_atom("_NET_CLIENT_LIST")
        self._CURRENT_DESKTOP = dsp.intern_atom("_NET_CURRENT_DESKTOP")
        self._DESKTOP = dsp.intern_atom("_NET_WM_DESKTOP")
        self._DESKTOP_COUNT = dsp.intern_atom("_NET_NUMBER_OF_DESKTOPS")
        self._DESKTOP_NAMES = dsp.intern_atom("_NET_DESKTOP_NAMES")
        self._HIDDEN = dsp.intern_atom("_NET_WM_STATE_HIDDEN")
        self._ICON = dsp.intern_atom("_NET_WM_ICON")
        self._NAME = dsp.intern_atom("_NET_WM_NAME")
        self._RPM = dsp.intern_atom("_XROOTPMAP_ID")
        self._SHADED = dsp.intern_atom("_NET_WM_STATE_SHADED")
        self._SHOWING_DESKTOP = dsp.intern_atom("_NET_SHOWING_DESKTOP")
        self._SKIP_PAGER = dsp.intern_atom("_NET_WM_STATE_SKIP_PAGER")
        self._SKIP_TASKBAR = dsp.intern_atom("_NET_WM_STATE_SKIP_TASKBAR")
        self._STATE = dsp.intern_atom("_NET_WM_STATE")
        self._STICKY = dsp.intern_atom("_NET_WM_STATE_STICKY")
        self._STRUT = dsp.intern_atom("_NET_WM_STRUT")
        self._STRUTP = dsp.intern_atom("_NET_WM_STRUT_PARTIAL")
        self._WMSTATE = dsp.intern_atom("WM_STATE")

    #------------------------------------------------
    def sendEvent(self, win, ctype, data, mask=None):
        #------------------------------------------------
        """ Send a ClientMessage event to the root """
        data = (data + [0] * (5 - len(data)))[:5]
        ev = Xlib.protocol.event.ClientMessage(
            window=win, client_type=ctype, data=(32, (data)))

        if not mask:
            mask = (X.SubstructureRedirectMask | X.SubstructureNotifyMask)
        self.root.send_event(ev, event_mask=mask)

    def create_menu(self):
        self.menu = Gtk.Menu()
        i1 = Gtk.MenuItem("Close All")
        i1.connect("activate", self.close_all)
        self.menu.append(i1)
        self.menu.show_all()

    def activateWindow(self, taskcat, offset=1):
        win = taskcat.windows[0]
        if win.id == ewmh.getActiveWindow().id:
            taskcat.windows = taskcat.windows[offset:] +\
                taskcat.windows[:offset]
            win = taskcat.windows[0]
        os.system('wmctrl -i -a %d' % win.id)
        # win.map()
        # ewmh.setActiveWindow(win)
        # ewmh.display.flush()

    def toggleWindow(self, taskcat):
        winids = [win.id for win in taskcat.windows]
        if ewmh.getActiveWindow().id in winids:
            for win in taskcat.windows:
                self.sendEvent(win, self.WM_CHANGE_STATE,
                               [Xutil.IconicState])
        else:
            for win in taskcat.windows:
                if self._NET_WM_STATE_HIDDEN not in ewmh.getWmState(win):
                    ewmh.setActiveWindow(win)
                    break
            else:
                for win in taskcat.windows:
                    win.map()
        ewmh.display.flush()
        self.display.flush()

    def getTask(self, event):
        index = event.x // self.size
        index = int(index)
        return self.tasks_draw[index]  # .windows[0]

    def tick(self):
        childLogger.debug('tick')
        draw = False
        self.display.sync()
        num = self.display.pending_events()
        childLogger.debug('events:%s', num)
        for _ in range(num):
            e = self.display.next_event()
            if e.type == X.MapNotify and e.window:
                if self.addTask(e.window.id):
                    draw = True
            elif e.type == X.DestroyNotify:
                if self.removeTask(e.window):
                    draw = True
        if draw:
            GObject.timeout_add(1000 // self.fps, self.update)
        return True  # Causes timeout to tick again.

    def do_draw_cb(self, widget, cr):
        w = self.size * sum([t.size for t in self.tasks_draw])
        h = self.size

        cr.set_source_rgba(*self.background_color)
        cr.rectangle(0, 0, w, h)
        cr.fill()
        # x, y = self.size / 2, self.size / 2
        x, y_center, w, h = 0, h / 2, h, h
        for task in self.tasks_draw:
            rel = task.size
            if rel <= 0:
                continue
            rel_x, rel_y = x / rel, y_center / rel
            if rel < 1:
                cr.scale(rel, rel)
            cr.set_source_surface(task.icon, rel_x, rel_y - h / 2)
            cr.get_source().set_filter(cairo.FILTER_FAST)
            if rel < 1:
                cr.identity_matrix()
            cr.paint()
            x += w * rel

    def update(self):
        draw = False
        for t in self.tasks_draw:
            if t.animate(1 / self.animation_duration / self.fps):
                draw = True
            elif t.destroy():
                self.tasks_draw.remove(t)
                t.icon.finish()
        if draw:
            w = self.size * sum([t.size for t in self.tasks_draw])
            h = self.size
            self.set_size_request(w, h)
            self.queue_draw()
            return True

    def removeTask(self, win):
        for t in self.tasks_draw:
            for w in t.windows:
                if win.id == w.id:
                    t.windows.remove(w)
                    return True

    def addTask(self, winid):
        """
        用wmclass寻找icon的话,mangameeya的icon无法读取,xdg文件中的qmlterm的icon页无法读取.docky都做得到.前者compiz做得到,后者做不到
        """
        win = self.display.create_resource_object("window", winid)
        try:
            lst = win.get_full_property(self._STATE, Xatom.ATOM).value
            if self._SKIP_TASKBAR in lst:
                return False
        except:
            return False

        try:
            for _ in range(10):
                wm_class = win.get_full_property(Xatom.WM_CLASS, Xatom.STRING)
                if wm_class:
                    wm_class = wm_class.value.decode().split("\0")[:2]
                    break
            else:
                return False
                wm_class = ["", ""]
        except:
            return False

        if self._NET_WM_WINDOW_TYPE_DOCK in ewmh.getWmWindowType(win):
            return False

        for taskcat in self.tasks_draw:
            if taskcat.wm_class == wm_class:
                for w in taskcat.windows:
                    if w.id == win.id:
                        return False
                taskcat.windows.append(win)
                return False
        else:
            for icon_name in wm_class:
                icon_name = icon_name.lower()
                if self.icontheme.has_icon(icon_name):
                    icon = self.icontheme.load_surface(icon_name, self.size,
                                                       1, self.get_window(), 0)
                    taskcat = TaskCat(self.display,  icon, wm_class)
                    taskcat.windows.append(win)
                    self.tasks_draw.append(taskcat)
                    mask = X.PropertyChangeMask | X.FocusChangeMask | X.StructureNotifyMask
                    # win.change_attributes(event_mask=mask)
                    return True
        return False


class TaskCat:

    def __init__(self, display,  icon, wm_class):
        self.windows = []
        self.icon = icon
        self.wm_class = wm_class
        self.size = 0

    def destroy(self):
        return len(self.windows) < 1 and self.size <= 0

    def animate(self, animate_step):
        if len(self.windows) < 1:
            if self.size > 0:
                self.size -= animate_step
                self.size = max(0, self.size)
                return True
        else:
            if self.size < 1:
                self.size += animate_step
                self.size = min(1, self.size)
                return True



import gi
gi.require_version('Gtk', '3.0')
import pyaudio
from logging import getLogger
from gi.repository import Gtk, Gdk
import cairo
from threading import Thread
import psutil
from . import helper

childLogger = getLogger(__name__)

vertical = True


class Multiload(Gtk.DrawingArea):
    stop = False

    def __init__(self,
                 background_color,
                 colors,
                 width, height,
                 fake_shadow=True,
                 grid=(4, 3),
                 inner_gap=2,
                 outer_gap=2,
                 layout=(2, 2),
                 interval=1,):
        super(Multiload, self).__init__()

        self.interval = interval
        self.colors = colors
        self.show_swap_mem = False
        self.background_color = background_color
        self.height = height
        self.width = width
        self.fake_shadow = fake_shadow
        self.grid = grid
        self.inner_gap = inner_gap
        self.outer_gap = outer_gap
        self.unit_width = (width - 2 * outer_gap) // layout[0] - inner_gap * 2
        self.unit_height = (
            height - 2 * outer_gap) // layout[1] - inner_gap * 2
        self.layout = layout

        self.override_background_color(Gtk.StateType.NORMAL, Gdk.RGBA(alpha=0))

        self.set_size_request(self.width, self.height)
        self.history = [None] * self.unit_width
        self.connect('draw', self.do_draw_cb)
        Thread(target=self.tick).start()

    def fetch_net(self):
        prev_net_io = psutil.net_io_counters(pernic=True)
        while True:
            net_io = psutil.net_io_counters(pernic=True)
            sent, recv = 0, 0
            for interface in net_io:
                if interface == 'lo':
                    continue
                sent += net_io[interface].bytes_sent - \
                    prev_net_io[interface].bytes_sent
                recv += net_io[interface].bytes_recv - \
                    prev_net_io[interface].bytes_recv
            yield sent, recv
            prev_net_io = net_io

    def fetch_mem(self):
        while True:
            vm = psutil.virtual_memory()
            total = vm.total
            if self.show_swap_mem:
                sm = psutil.swap_memory()
                total = sm.total
            result = [vm.used / total]
            if self.show_swap_mem:
                result.append(sm.used / total)
            yield result

    def fetch_disk(self):
        prev_disk_io = psutil.disk_io_counters(perdisk=False)
        while True:
            disk_io = psutil.disk_io_counters(perdisk=False)
            yield disk_io.write_bytes - prev_disk_io.write_bytes, disk_io.read_bytes - prev_disk_io.read_bytes
            prev_disk_io = disk_io

    def tick(self):
        net_data = self.fetch_net()
        mem_data = self.fetch_mem()
        disk_data = self.fetch_disk()
        while not self.stop:
            childLogger.debug('tick')
            self.history.pop(0)
            cpu = psutil.cpu_percent(self.interval, percpu=True)
            cpu = [c / 100 / len(cpu) for c in cpu]
            data = {
                "cpu": cpu,
                'net': next(net_data),
                'disk': next(disk_data),
                'mem': next(mem_data),
            }
            childLogger.debug('Multiload data:%s', data)
            self.history.append(data)
            self.queue_draw()

    def destory(self):
        childLogger.debug('stop')
        self.stop = True

    def do_draw_cb(self, widget, cr):
        cr.set_source_rgba(*self.background_color)
        cr.rectangle(0, 0, self.width, self.height)
        cr.fill()

        childLogger.debug('cairo start')
        if not self.history[-1]:
            return

        max_net = 1 + max([sum(item['net']) for item in self.history if item])
        max_disk = 1 + max([sum(item['disk'])
                            for item in self.history if item])
        postions = []
        for x in range(self.layout[0]):
            for y in range(self.layout[1]):
                _x = self.outer_gap + x * \
                    (self.unit_width + 2 * self.inner_gap)
                _y = self.outer_gap + y * \
                    (self.unit_height + 2 * self.inner_gap)
                postions.append((_x, _y))

        for unit, max_value, position in zip(
                ['cpu', 'mem', 'net',  'disk'],
                [1, 1, max_net,  max_disk],
                postions):
            x, y = position
            self.draw_unit(cr, x + self.inner_gap, y +
                           self.inner_gap, unit,  max_value)
        childLogger.debug('cairo end')

    def draw_unit(self, cr, x, y, unit,   max_value=1):
        colors = self.colors[unit]['foreground']
        back = self.colors[unit]['background']

        cr.set_operator(cairo.OPERATOR_SOURCE)
        cr.set_source_rgba(*helper.color(back))
        cr.rectangle(x, y, self.unit_width, self.unit_height)
        cr.fill()

        if self.grid:
            cr.set_line_width(1)
            cr.set_source_rgba(1, 1, 1, 0.3)
            for i in range(1, self.grid[0]):
                _x = -.5 + int(x + self.unit_width / self.grid[0] * i)
                cr.move_to(_x, -.5 + y)
                cr.line_to(_x, -.5 + y + self.unit_height)
            for i in range(1, self.grid[1]):
                _y = -.5 + int(y + self.unit_height / self.grid[1] * i)
                cr.move_to(-.5 + x, _y)
                cr.line_to(-.5 + x + self.unit_width, _y)
            cr.stroke()

        cr.set_operator(cairo.OPERATOR_OVER)
        line_count = len(self.history[-1][unit])
        for num_line in range(line_count):
            cr.set_source_rgba(
                *helper.color(colors[num_line % len(self.colors)]))
            cr.move_to(x, y + self.unit_height)
            self.draw_line(x, y, cr, lambda item: sum(
                item[unit][num_line:]) / max_value)
            self.draw_line(x, y, cr, 'bottom', True)
            cr.close_path()
            cr.fill()
        if self.fake_shadow:
            cr.set_line_width(1)
            cr.set_source_rgba(1, 1, 1, 1)
            cr.move_to(-.5 + x + self.unit_width, -.5 + y)
            cr.line_to(
                -.5 + x + self.unit_width,
                -.5 + y + self.unit_height)
            cr.line_to(-.5 + x, -.5 + y + self.unit_height)
            cr.stroke()
            cr.set_source_rgba(0, 0, 0, 1)
            cr.move_to(-.5 + x + self.unit_width, -.5 + y)
            cr.line_to(-.5 + x, -.5 + y)
            cr.line_to(-.5 + x, -.5 + y + self.unit_height)
            cr.stroke()
        return line_count

    def draw_line(self, x, y, cr, calc, reverse=False):
        if calc in ('top', 'bottom'):
            h = 0 if calc == 'top' else self.unit_height
            start = self.unit_width if reverse else 0
            end = 0 if reverse else self.unit_height
            cr.line_to(x + start, y + h)
            cr.line_to(x + end, y + h)
        else:
            data = self.history[::-1 if reverse else 1]
            for w, item in enumerate(data):
                h = calc(item) if item else 0
                h = self.unit_height * (1 - h)
                cr.line_to(x + w, y + h)



#!/usr/bin/python
log = 'info'
height = 32
position = 'bottom'
background_color = '#ffffff30'
fake_shadow = True

style_sheet="""
.fake-shadow-top{
    border-top:solid 1px white;
}
.fake-shadow-bottom{
    border-bottom:solid 1px black;
}
.label{
color:black;
padding-left:5px;
padding-right:5px;
text-shadow: 1px 1px 5px black; 
}
"""

sections = [
#    {
#        # mpd
#        'auto-command': 'mpc | head -n 1',
#        'interval': 1,
#        'max-width': 20,
#        'click': 'mpc toggle',
#        'scroll-up': 'mpc prev',
#        'scroll-down': 'mpc next',
#    },
    'taskbar',
    'visualizer',
    {
        # volume control
        'icon-name': 'preferences-system-sound',
        'scroll-up': 'pamixer -u;pamixer -i 1;',
        'scroll-down': 'pamixer -u;pamixer -d 1;',
        'click': 'pamixer -t',
    }, {
        # show desktop
        'icon-name': 'desktop',
        'click': """
        if wmctrl -m | grep "mode: ON";then
            wmctrl -k off
        else
            wmctrl -k on
        fi""",
    },
    'multiload',
    {
        # time
        'auto-command': 'date +%T',
        'interval': 1,
    },
]
 
visualizer_background = background_color
visualizer_padding = height / 8
visualizer_decay = 0.01

multiload_interval = 0.5
multiload_layout = 4, 1,
multiload_inner_gap = 2
multiload_outer_gap = 2
multiload_fake_shadow = True
multiload_cpu_background = '#00000010'
multiload_cpu_foreground = '#00ffffff', '#ff00ffff', '#0000ffff', '#00ff00ff',  '#008080ff', '#0080ffff',
multiload_mem_background = multiload_cpu_background
multiload_mem_foreground = '#00ff00ff', '#ff00ffff', 
multiload_net_background = multiload_cpu_background
multiload_net_foreground = '#f08000ff', '#0080ffff',
multiload_disk_background = multiload_cpu_background
multiload_disk_foreground = '#808000ff', '#008080ff',

# overwrite default configuration
import os.path

config_file = os.path.expanduser("~/.config/panonrc")
try:
    s = open(config_file).read()
    exec(s)
except BaseException:
    pass



import pyaudio


class Source:
    def __init__(self,  channel_count, sample_rate):
        self.channel_count = channel_count
        self.sample_rate = sample_rate

        self.start()

    def read(self):
        size = self.stream.get_read_available()
        return self.stream.read(size)

    def stop(self):
        self.stream.close()

    def start(self):
        p = pyaudio.PyAudio()
        self.stream = p.open(format=pyaudio.paInt16,
                             channels=self.channel_count,
                             rate=self.sample_rate,
                             input=True)



import os
import sys
import json
import socket
from threading import Thread
import logging


class Singleton:

    def __init__(self,  socket_file, timeout=0.01):
        self.socket_file = socket_file
        self.timeout = timeout
        self.socket = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
        self.stop = False

    def server(self):
        try:
            self.socket.bind(self.socket_file)
        except OSError:
            return False
        logging.debug('listen')
        self.socket.listen(1)
        return True

    def loop(self):
        self.socket.settimeout(self.timeout)
        while not self.stop:
            try:
                conn, addr = self.socket.accept()
            except KeyboardInterrupt:
                break
            except socket.timeout:
                yield None
                continue

            try:
                logging.debug('new obj')
                conn.send(b'welcome')
                obj = b''
                while True:
                    data = conn.recv(1024)
                    if data is None or len(data) < 1:
                        break
                    if len(data) < 1:
                        break
                    obj += data
                logging.debug('new json obj')
                conn.close()
                yield json.loads(obj.decode())
            except:
                yield None

    def finish(self):
        self.stop = True
        os.remove(self.socket_file)

    def client(self):
        if not os.path.exists(self.socket_file):
            return False
        self.socket.settimeout(0.5)
        try:
            self.socket.connect(self.socket_file)
        except:
            return False
        msg = self.socket.recv(1024)
        if not msg == b'welcome':
            return False
        env = {k: v for k, v in os.environ.items()}
        obj = {'argv': sys.argv, 'env': env}
        obj = json.dumps(obj).encode()
        self.socket.send(obj)
        self.socket.close()
        return True

    def start(self):
        if self.client():
            return False
        else:
            if os.path.exists(self.socket_file):
                os.remove(self.socket_file)
            self.server()
            return True


if __name__ == '__main__':
    logging.basicConfig(level=logging.DEBUG,)
    SOCKET_FILE = "/run/user/%s/desktop_menu.socket" % os.getuid()
    sin = Singleton(SOCKET_FILE)
    if sin.start():
        for obj in sin.loop():
            print(obj)
        sin.finish()
        print('end loop')



from .. import glsl
import ModernGL


class GLFFT:
    def __init__(self):
        ctx = ModernGL.create_standalone_context()
        compute_shader = ctx.compute_shader(glsl.load('fft.glsl'))
        size = 1024 * 4
        empty = b'\00' * size * 4
        buf1 = ctx.buffer(empty)
        empty = b'\00' * size * 4
        buf2 = ctx.buffer(empty)
        buf1.bind_to_storage_buffer(1)
        buf2.bind_to_storage_buffer(2)
        #compute_shader.uniforms['mul'].value = 100.0

        self.compute_shader = compute_shader
        #self.compute_shader.uniforms['mul'].value = 100.0
        self.ctx = ctx
        self.buf1 = buf1
        self.buf2 = buf2

    def compute(self, data):
        data=data[-1024*4*4:]
        self.compute_shader.uniforms['real_size'].value = len(data)//4
        #self.compute_shader.uniforms['mul'].value = 100.0
        self.buf1.write(data)
        self.compute_shader.run()
        return self.buf2.read()[:len(data)]

    def destroy(self):
        self.ctx.release()



from setuptools import setup
import glob

setup(
    name='panon',
    version='v0.1.1',
    description='A status bar for X window managers',
    url='http://github.com/rbn42/panon',
    download_url='https://github.com/rbn42/panon/archive/v0.1.1.tar.gz',
    author='rbn42',
    author_email='bl100@students.waikato.ac.nz',
    license='MIT',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Environment :: X11 Applications',
        'Intended Audience :: End Users/Desktop',
        'License :: OSI Approved :: GNU Affero General Public License v3',
        'Programming Language :: Python :: 3',
    ],
    keywords=['visualizer', 'multiload', 'panel', 'xorg'],
    packages=['panon','panon.visualizer','panon.glsl'],    
    package_data={ 
        'panon.glsl': ['*.vert','*.frag', "*.glsl"],
    },        
    install_requires=[
        #'docopt',
        'ewmh',
        'numpy',
        'pyaudio',
        'psutil',
        'python-xlib',
    ],
    data_files=[
        ('share/doc/panon/config', glob.glob('doc/config/*')),
    ],
    entry_points={
        'console_scripts': [
            'panon=panon.__main__:main',
        ]
    }
)
